xdel(winsid());

A = [];
B = [];
X = [];

w = 600;
h = 400;
f = figure('position',[100 100 w h],'figure_name','Wavetable Generator');
toolbar(f.figure_id,'off');

A_axis = newaxes();
A_axis.margins=[0 0 0 0];
A_axis.axes_bounds=[0,0,0.5,0.4];
A_axis.auto_clear='off';

B_axis = newaxes();
B_axis.margins=[0 0 0 0];
B_axis.axes_bounds=[0.5,0,0.5,0.4];
B_axis.auto_clear='off';

uicontrol("style","pushbutton","string","Load Wave A","FontSize",16,"units","normalized","position",[0.15,0.4,0.25,0.167],"callback","A=load_cycle(A_axis,A); X = []");
uicontrol("style","pushbutton","string","Load Wave B","FontSize",16,"units","normalized","position",[0.6,0.4,0.25,0.167],"callback","B=load_cycle(B_axis,B); X = []");
save_button = uicontrol("style","pushbutton","string","Save","FontSize",16,"units","normalized","position",[0.425,0.4,0.15,0.167],"enable","off","callback","save_result()");

uicontrol("style","frame","relief","groove","units","normalized","position",[0.375,0.035,0.6,0.3]);
uicontrol("style","pushbutton","string","Audition","FontSize",20,"units","normalized","position",[0.4,0.06,0.425,0.25],"callback","X = wave_audition()");
octave_list = uicontrol("style","popupmenu","relief","sunken","units","normalized","position",[0.85,0.14,0.1,0.05],"string",["4" "3" "2" "1" "0" "-1" "-2" "-3" "-4"],"value",5);
uicontrol("style","text","string","Ocatve","units","normalized","position",[0.85,0.20,0.1,0.05]);

uicontrol("style","text","string","Power Bands","horizontalalignment","left","units","normalized","position",[0.05,0.28,0.15,0.04]);
cycle_bands_box = uicontrol("style","edit","string","32","horizontalalignment","right","units","normalized","position",[0.2,0.28,0.15,0.05],"callback","edited(); X = []");

uicontrol("style","text","string","Cycle Length","horizontalalignment","left","units","normalized","position",[0.05,0.22,0.15,0.04]);
cycle_length_box = uicontrol("style","edit","string","256","horizontalalignment","right","units","normalized","position",[0.2,0.22,0.15,0.05],"callback","parameter_edit(); X = []");

uicontrol("style","text","string","Cycle Count","horizontalalignment","left","units","normalized","position",[0.05,0.16,0.15,0.04]);
cycle_count_box = uicontrol("style","edit","string","100","horizontalalignment","right","units","normalized","position",[0.2,0.16,0.15,0.05],"callback","parameter_edit(); X = []");

uicontrol("style","text","string","Total Length","horizontalalignment","left","units","normalized","position",[0.05,0.1,0.15,0.04]);
total_length_box = uicontrol("style","edit","string","25600","horizontalalignment","right","units","normalized","position",[0.2,0.1,0.15,0.05],"enable","off");

uicontrol("style","text","string","Blend Method","horizontalalignment","left","units","normalized","position",[0.05,0.04,0.15,0.04]);
mode_menu = uicontrol("style","popupmenu","relief","sunken","string",["raised cosine","FFT crossfade"],"units","normalized","position",[0.2,0.04,0.15,0.05],"callback","edited(); X = []");

uicontrol("style","text","string","Function","units","normalized","position",[0.03,0.5,0.1,0.04]);
wave_menu_a = uicontrol("style","popupmenu","relief","sunken","string",["invert wave","reverse time","remove even harmonics","remove odd harmonics","force even symmetry","force odd symmetry"],"units","normalized","position",[0.03,0.44,0.1,0.05],"callback","A = wave_edit(A,get(wave_menu_a,''value''),A_axis); X = []");
uicontrol("style","text","string","Function","units","normalized","position",[0.87,0.5,0.1,0.04]);
wave_menu_b = uicontrol("style","popupmenu","relief","sunken","string",["invert wave","reverse time","remove even harmonics","remove odd harmonics","force even symmetry","force odd symmetry"],"units","normalized","position",[0.87,0.44,0.1,0.05],"callback","B = wave_edit(B,get(wave_menu_b,''value''),B_axis); X = []");

function wave_out = load_cycle(plot_axis,wave_in)
    sca(plot_axis);
    PathFileName = uigetfile("*.wav","","Select Waveform");
    if (strindex(convstr(PathFileName,'l'),'.wav') ~= []) then
        wave_out = wavread(PathFileName);
        wave_out = wave_out/max(abs(wave_out));
        drawlater();
        plot_axis.auto_clear='on'
        plot(wave_out);
        plot_axis.tight_limits='on';
        plot_axis.axes_visible=['off','off','off'];
        plot_axis.auto_clear='off'
        drawnow();
        edited();
    else
        wave_out = wave_in;
    end
endfunction
    
function save_result()
    PathFileName = uiputfile(".wav","","Output File");
    if (strindex(convstr(PathFileName,'l'),'.wav') ~= []) then
        wavwrite(X,47863*2^(5-get(octave_list,"value")),16,PathFileName);
    end
endfunction

function parameter_edit()
    total = eval(get(cycle_length_box,"string"))*eval(get(cycle_count_box,"string"));
    set(total_length_box,"string",string(total))
    if (total > 500000) then
        set(total_length_box,"BackgroundColor",[0.8,0.4,0.4]);
    else
        set(total_length_box,"BackgroundColor",[0.8,0.8,0.8]);
    end
    edited();
endfunction

function edited()
    set(save_button,"enable","off");
endfunction

function wave_out = wave_audition()
    if ((A ~= []) & (B ~= [])) then
        if (X == []) then
            if (get(mode_menu,"value") == 2) then
                X = crossfade();
            else
                X = granulate();
            end
            set(save_button,"enable","on");
        end
        playsnd(X,48000*2^(5-get(octave_list,"value")));
        wave_out = X;
    else
        set(save_button,"enable","off");
        messagebox("Requires 2 waveforms to process","Error","error");
        wave_out = [];
    end
endfunction

function wave_out = granulate()
grains = eval(get(cycle_bands_box,"string"));
segments = eval(get(cycle_count_box,"string"));
output_length = eval(get(cycle_length_box,"string"));

plot_axis = sca(A_axis);

ra = length(A);
rb = length(B);

a_est = A;
a_widths = zeros(1,grains);
a_scales = zeros(1,grains);
a_locations = zeros(1,grains);
w = waitbar("Processing Input Waves");
for j = 1:grains
    peak_error = 1;
    peak_width = 0;
    peak_scale = 0;
    peak_index = 0;
    for k = 2:(ra/2) //width
        blip = 1 - cos(2*%pi*(1:k)/k);
        spread = conv([zeros(1,k),a_est,zeros(1,k)],blip,"same");
        squash = [spread((k+1):(k+k))+spread((ra+k+1):(ra+k+k)),spread((k+k+1):(ra)),spread((ra+1):(ra+k))+spread(1:k)];
        [top ind] = max(abs(squash));
        blank = zeros(1,ra);
        blank(ind) = 1;
        spread = conv([zeros(1,k),blank,zeros(1,k)],blip,"same");
        squash = [spread((k+1):(k+k))+spread((ra+k+1):(ra+k+k)),spread((k+k+1):(ra)),spread((ra+1):(ra+k))+spread(1:k)];
        subbed_error = 1;
        subbed_scale = 0;
        for l = -1:0.1:1
            if (mean((a_est-squash*l).^2) < subbed_error) then
                subbed_scale = l;
                subbed_error = mean((a_est-squash*l).^2);
            end
        end
        for l = (subbed_scale-0.1):0.01:(subbed_scale+0.1)
            if (mean((a_est-squash*l).^2) < subbed_error) then
                subbed_scale = l;
                subbed_error = mean((a_est-squash*l).^2);
            end
        end
        if (mean((a_est-squash*subbed_scale).^2) < peak_error) then
            peak_error = mean((a_est-squash*subbed_scale).^2);
            peak_width = k;
            peak_scale = subbed_scale;
            peak_index = ind;
        end
    end
    
    blip = 1 - cos(2*%pi*(1:peak_width)/peak_width);
    blank = zeros(1,ra);
    blank(peak_index) = 1;
    spread = conv([zeros(1,peak_width),blank,zeros(1,peak_width)],blip,"same");
    squash = [spread((peak_width+1):(peak_width+peak_width))+spread((ra+peak_width+1):(ra+peak_width+peak_width)),spread((peak_width+peak_width+1):(ra)),spread((ra+1):(ra+peak_width))+spread(1:peak_width)];
    drawlater();
    plot(a_est,'r');
    plot(A,'b');
    plot(squash*peak_scale,'g');
    plot_axis.tight_limits='on';
    plot_axis.axes_visible=['off','off','off'];
    drawnow();
    a_est = a_est - squash*peak_scale;
    
    a_widths(j) = peak_width/ra;
    a_scales(j) = peak_scale;
    a_locations(j) = peak_index/ra;
    waitbar(j/(2*grains),w);
end

drawlater();
plot_axis.auto_clear='on'
plot(A-a_est,'g');
plot_axis.auto_clear='off'
plot(A,'b');
plot_axis.tight_limits='on';
plot_axis.axes_visible=['off','off','off'];
drawnow();
plot_axis = sca(B_axis);

b_est = B;
b_widths = zeros(1,grains);
b_scales = zeros(1,grains);
b_locations = zeros(1,grains);

for j = 1:grains
    peak_error = 1;
    peak_width = 0;
    peak_scale = 0;
    peak_index = 0;
    for k = 2:(rb/2) //width
        blip = 1 - cos(2*%pi*(1:k)/k);
        spread = conv([zeros(1,k),b_est,zeros(1,k)],blip,"same");
        squash = [spread((k+1):(k+k))+spread((rb+k+1):(rb+k+k)),spread((k+k+1):(rb)),spread((rb+1):(rb+k))+spread(1:k)];
        [top ind] = max(abs(squash));
        blank = zeros(1,rb);
        blank(ind) = 1;
        spread = conv([zeros(1,k),blank,zeros(1,k)],blip,"same");
        squash = [spread((k+1):(k+k))+spread((rb+k+1):(rb+k+k)),spread((k+k+1):(rb)),spread((rb+1):(rb+k))+spread(1:k)];
        subbed_error = 1;
        subbed_scale = 0;
        for l = -1:0.1:1
            if (mean((b_est-squash*l).^2) < subbed_error) then
                subbed_scale = l;
                subbed_error = mean((b_est-squash*l).^2);
            end
        end
        for l = (subbed_scale-0.1):0.01:(subbed_scale+0.1)
            if (mean((b_est-squash*l).^2) < subbed_error) then
                subbed_scale = l;
                subbed_error = mean((b_est-squash*l).^2);
            end
        end
        if (mean((b_est-squash*subbed_scale).^2) < peak_error) then
            peak_error = mean((b_est-squash*subbed_scale).^2);
            peak_width = k;
            peak_scale = subbed_scale;
            peak_index = ind;
        end
    end
    
    blip = 1 - cos(2*%pi*(1:peak_width)/peak_width);
    blank = zeros(1,rb);
    blank(peak_index) = 1;
    spread = conv([zeros(1,peak_width),blank,zeros(1,peak_width)],blip,"same");
    squash = [spread((peak_width+1):(peak_width+peak_width))+spread((rb+peak_width+1):(rb+peak_width+peak_width)),spread((peak_width+peak_width+1):(rb)),spread((rb+1):(rb+peak_width))+spread(1:peak_width)];
    drawlater();
    plot(b_est,'r');
    plot(B,'b');
    plot(squash*peak_scale,'g');
    plot_axis.tight_limits='on';
    plot_axis.axes_visible=['off','off','off'];
    drawnow();
    b_est = b_est - squash*peak_scale;
    
    b_widths(j) = peak_width/rb;
    b_scales(j) = peak_scale;
    b_locations(j) = peak_index/rb;
    waitbar((j+grains)/(2*grains),w);
end

plot_axis = sca(B_axis);
plot_axis.auto_clear='on'

deltas = b_locations - a_locations;
deltas = deltas - (deltas > 0.5) + (deltas < -0.5);

X = zeros(1,(segments+1)*output_length);
for i = 0:segments
    weight = i/segments;
    z_est = zeros(1,output_length);
    for j = 1:grains
        peak_width = round(((1-weight)*a_widths(j)+weight*b_widths(j))*output_length);
        peak_index = round((a_locations(j)+weight*(deltas(j)))*output_length);
        peak_index = peak_index + (peak_index < 1)*output_length - (peak_index > output_length)*output_length;
        blip = 1 - cos(2*%pi*(1:peak_width)/peak_width);
        blank = zeros(1,output_length);
        blank(peak_index) = 1;
        spread = conv([zeros(1,peak_width),blank,zeros(1,peak_width)],blip,"same");
        squash = [spread((peak_width+1):(peak_width+peak_width))+spread((output_length+peak_width+1):(output_length+peak_width+peak_width)),spread((peak_width+peak_width+1):(output_length)),spread((output_length+1):(output_length+peak_width))+spread(1:peak_width)];
        z_est = z_est + squash*((1-weight)*a_scales(j)+weight*b_scales(j));
    end
    drawlater();
    plot(z_est);
    plot_axis.tight_limits='on';
    plot_axis.axes_visible=['off','off','off'];
    drawnow();
    X((i*output_length+1):((i+1)*output_length)) = z_est/max(abs(z_est));
    waitbar(i/segments,"Creating Output",w);
end

close(w);
drawlater();
plot_axis.auto_clear='on'
plot(B-b_est,'g');
plot_axis.auto_clear='off'
plot(B,'b');
plot_axis.tight_limits='on';
plot_axis.axes_visible=['off','off','off'];
drawnow();

wave_out = X;
endfunction

function wave_out = crossfade()
grains = eval(get(cycle_bands_box,"string"));
segments = eval(get(cycle_count_box,"string"));
output_length = eval(get(cycle_length_box,"string"));

ra = length(A);
rb = length(B);
a_fft = zeros(1,output_length);
temp = fft(A,-1);
a_fft(1:min(ra,output_length,grains)) = temp(1:min(ra,output_length,grains));

a_est = zeros(1,max(output_length,ra));
a_est(1:min(ra,output_length,grains)) = temp(1:min(ra,output_length,grains));
temp = real(fft(a_est(1:ra),1));
plot_axis = sca(A_axis);
drawlater();
plot_axis.auto_clear='on'
plot(temp/max(abs(temp)),'g');
plot_axis.auto_clear='off'
plot(A,'b');
plot_axis.tight_limits='on';
plot_axis.axes_visible=['off','off','off'];
drawnow();

b_fft = zeros(1,output_length);
temp = fft(B,-1);
b_fft(1:min(rb,output_length,grains)) = temp(1:min(rb,output_length,grains));

b_est = zeros(1,max(output_length,rb));
b_est(1:min(rb,output_length,grains)) = temp(1:min(rb,output_length,grains));

X = zeros(1,(segments+1)*output_length);

plot_axis = sca(B_axis);
plot_axis.auto_clear='on';
w = waitbar("Creating Output");

for i = 0:segments
    weight = i/segments;
    z_est = real(fft((1-weight)*a_fft+weight*b_fft,1));
    drawlater();
    plot(z_est);
    plot_axis.tight_limits='on';
    plot_axis.axes_visible=['off','off','off'];
    drawnow();
    X((i*output_length+1):((i+1)*output_length)) = z_est/max(abs(z_est));
    waitbar(i/segments,"Creating Output",w);
end

close(w);
temp = real(fft(b_est(1:rb),1));
drawlater();
plot_axis.auto_clear='on';
plot(temp/max(abs(temp)),'g');
plot_axis.auto_clear='off'
plot(B,'b');
plot_axis.tight_limits='on';
plot_axis.axes_visible=['off','off','off'];
drawnow();

wave_out = X;
endfunction

function wave_out = wave_edit(wave_in,index_in,axis_in)
    select index_in
    case 1
        wave_out = -wave_in;
    case 2
        wave_out = wave_in(length(wave_in):-1:1);
    case 3
        temp = 1-pmodulo(1:length(wave_in),2);
        temp(1) = 1;
        temp = temp .* fft(wave_in,-1);
        wave_out = real(fft(temp,1));
    case 4
        temp = pmodulo(1:length(wave_in),2);
        temp(2) = 1;
        temp = temp .* fft(wave_in,-1);
        wave_out = real(fft(temp,1));
    case 5
        temp = length(wave_in);
        temp = ((1:temp) - 1)/(temp -1);
        wave_out = wave_in(length(wave_in):-1:1) .* temp + wave_in .* (1-temp);
    case 6
        temp = length(wave_in);
        temp = ((1:temp) - 1)/(temp -1);
        wave_out = -wave_in(length(wave_in):-1:1) .* temp + wave_in .* (1-temp);
    end
    wave_out = wave_out/max(abs(wave_out));
    edited();
    plot_axis = sca(axis_in);
    drawlater();
    plot_axis.auto_clear='on'
    plot(wave_out);
    plot_axis.tight_limits='on';
    plot_axis.axes_visible=['off','off','off'];
    plot_axis.auto_clear='off'
    drawnow();
endfunction
